from lxml import html, etree
from parslepy import Parselet
from parslepy.funcs import xpathstrip, xpathtostring
from cssselect import HTMLTranslator
import re


ns = etree.FunctionNamespace(None)
ns['strip'] = xpathstrip
ns['str'] = xpathtostring


def extract_urls(self, xpath=None):
    urlpattern = re.compile(
        r'^(?:http)s?://'  # http:// or https://
        r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'  # domain...
        r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'  # ...or ip
        r'(?::\d+)?'  # optional port
        r'(?:/?|[/?]\S+)$', re.IGNORECASE)
    if xpath and not xpath.endswith('/'):
        xpath += '/'
    elif xpath is None:
        xpath = ""
    return set(url for url in self.xpath(xpath + "descendant-or-self::a/@href")
               if urlpattern.match(url))


def extract_text(self):
    return "".join(i.strip() for i in self.itertext())


def extract(self, rules, strict=False):
    parselet = Parselet(rules, strict=strict)
    return parselet.extract(self)


def cssselect(self, expr):
    return self._css_translator.css_to_xpath(expr)


def css(self, expr):
    return self.xpath(self.cssselect(expr))


def html_content(self, pretty_print=False):
    return html.tostring(self, pretty_print, encoding='utf-8')


html.HtmlElement.extract_text = extract_text
html.HtmlElement._css_translator = HTMLTranslator()
html.HtmlElement.cssselect = cssselect
html.HtmlElement.css = css
html.HtmlElement.extract = extract
html.HtmlElement.extract_urls = extract_urls
html.HtmlElement.html = html_content


def HtmlParser(response, absolute_links=True):
    """
    :param response:
    :type response: :class:`dragline.http.Response`

    This method takes response object as its argument and returns
    the lxml etree object.

    HtmlParser function returns a lxml object of type HtmlElement which got few potential methods.
    All the details of lxml object are discussed in section `lxml.html.HtmlElement`.
    """
    parser = html.HTMLParser(recover=True, encoding='utf-8')
    if isinstance(response, basestring):
        element = html.fromstring(response, None, parser)
    else:
        element = html.fromstring(response.body, response.url, parser)
        if absolute_links:
            element.make_links_absolute()
    return element



from requests.compat import urlencode, urldefrag
import socket
from hashlib import sha1
import time
import requests
from .defaultsettings import RequestSettings
from collections import defaultdict
import operator
import random
from .redisds import Dict
import types
import re

socket.setdefaulttimeout(300)


class RequestError(Exception):

    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class Request(object):

    """
    :param url: the URL of this request
    :type url: string
    :param method: the HTTP method of this request. Defaults to ``'GET'``.
    :type method: string
    :param headers: the headers of this request.
    :type headers: dict
    :param callback: name of the function to call after url is downloaded.
    :type callback: string
    :param meta:  A dict that contains arbitrary metadata for this request.
    :type meta: dict
    """

    settings = RequestSettings()
    stats = Dict("stats:*")
    method = "GET"
    form_data = None
    headers = {}
    callback = None
    meta = None
    retry = 0
    cookies = None
    callback_object = None
    dontfilter = False
    session = requests.Session()
    timeout = None
    fromcache = True
    proxy = []
    _cookie_regex = re.compile('(([^ =]*)?=[^ =]*?;)')

    def __init__(self, url, method=None, form_data=None, headers=None, callback=None, meta=None,
                 cookies=None, proxy=None, timeout=None, dontfilter=None, fromcache=None):
        if isinstance(url, str):
            self.url = str(url)
        elif isinstance(url, unicode):
            self.url = unicode(url)
        else:
            AssertionError("Invalid url type")
        if form_data:
            self.method = 'POST'
            self.form_data = form_data
        if method:
            assert method in ['GET', 'POST', 'HEAD', 'PUT', 'DELETE'], 'INVALID METHOD'
            self.method = method
        if callback:
            self.callback = callback
        if meta:
            self.meta = meta
        if headers:
            self.headers = headers
        if proxy:
            self.proxy = proxy
        if cookies:
            self.cookies = cookies
        if dontfilter:
            self.dontfilter = True
        if timeout:
            self.timeout = timeout
        if fromcache:
            self.fromcache = fromcache

    def __getstate__(self):
        d = self.__dict__.copy()
        if isinstance(self.callback, types.MethodType) and hasattr(self.callback, 'im_self'):
            d['callback'] = self.callback.__name__
            if not Request.callback_object == self.callback.im_self:
                Request.callback_object = self.callback.im_self
        return d

    def __setstate__(self, d):
        if 'callback' in d and isinstance(d['callback'], str):
            d['callback'] = getattr(Request.callback_object, d['callback'])
        self.__dict__ = d

    def __repr__(self):
        return "<%s>" % self.get_unique_id()

    def __str__(self):
        return self.get_unique_id()

    def __usha1(self, x):
        """sha1 with unicode support"""
        if isinstance(x, unicode):
            return sha1(x.encode('utf-8')).hexdigest()
        else:
            return sha1(x).hexdigest()

    def send(self):
        """
        This function sends HTTP requests.

        :returns: response
        :rtype: :class:`dragline.http.Response`
        :raises: :exc:`dragline.http.RequestError`: when failed to fetch contents

        >>> req = Request("http://www.example.org")
        >>> response = req.send()
        >>> print response.headers['status']
        200

        """

        try:
            if self.fromcache and self.settings.CACHE:
                cache = self.settings.CACHE[self.get_unique_id()]
                if cache is not None:
                    cache.fromcache = True
                    return cache
            if self.timeout:
                timeout = self.timeout
            else:
                timeout = max(self.settings.DELAY, self.settings.TIMEOUT)
            args = dict(url=self.url, method=self.method, data=self.form_data,
                        verify=False, timeout=timeout, cookies=self.cookies)
            if len(self.proxy) > 0:
                proxy = self.proxy
            elif len(self.settings.PROXIES) > 0:
                proxy = random.choice(self.settings.PROXIES)
            else:
                proxy = None
            if proxy:
                pattern = "http://%s:%s" if len(proxy) == 2 else "http://%s:%s@%s:%s"
                args['proxies'] = {"http": pattern % proxy, "https": pattern % proxy}
            args['headers'] = self.settings.HEADERS
            args['headers'].update(self.headers)
            res = Response(self.session.request(**args), self.meta)
            if self.settings.CACHE:
                self.settings.CACHE[self.get_unique_id()] = res

            if self.settings.AUTOTHROTTLE:
                self.updatedelay(res.elapsed)
                time.sleep(self.settings.DELAY)
        except Exception as e:
            raise RequestError(e.message)
        else:
            self.stats.inc('pages_crawled')
            self.stats.inc('request_bytes', len(res))
        return res

    def get_unique_id(self, hashing=False):
        request = self.method + ":" + urldefrag(self.url)[0]
        if self.form_data:
            request += ":" + urlencode(sorted(self.form_data.items(),
                                              key=operator.itemgetter(1)))
        if hashing:
            return self.__usha1(request)
        else:
            return request

    @classmethod
    def updatedelay(cls, delay):
        cls.settings.DELAY = min(
            max(cls.settings.MIN_DELAY, delay,
                (cls.settings.DELAY + delay) / 2.0),
            cls.settings.MAX_DELAY)


class Response(requests.Response):

    """
    This function is used to create user defined
    response to test your spider and also in many
    other cases.

    :param url: the URL of this response
    :type url: string

    :param headers: the headers of this response.
    :type headers: dict

    :param body: the response body.
    :type body: str

    :param meta: meta copied from request
    :type meta: dict

    """
    meta = None
    fromcache = False

    def __init__(self, response=requests.Response(), meta=None):
        if isinstance(response, requests.Response):
            self.__dict__ = response.__dict__
        if meta:
            self.meta = meta

    def __len__(self):
        if 'content-length' in self.headers:
            return int(self.headers['content-length'])
        return len(self.text)

    @property
    def body(self):
        return self.text

    @property
    def status(self):
        return self.status_code



from dragline import __version__
from gevent import spawn, joinall, monkey
monkey.patch_all()
import sys
import argparse
import os
import traceback
from .defaultsettings import SpiderSettings, LogSettings
from .crawl import Crawler


logger = LogSettings().getLogger("dragline")


def load_modules(path, spiderfile="main", settingsfile="settings"):
    try:
        spiderfile = spiderfile.strip('.py')
        settingsfile = settingsfile.strip('.py')
        sys.path.insert(0, path)
        spider = __import__(spiderfile)
        settings = __import__(settingsfile)
        del sys.path[0]
        return spider, settings
    except Exception as e:
        logger.exception("Failed to load module %s" % spiderfile)
        raise ImportError


def main(spider_class, settings_module):
    crawler = Crawler(spider_class, settings_module)
    try:
        joinall([spawn(crawler.process_url) for i in xrange(5)])
    except KeyboardInterrupt:
        crawler.clear(False)
    except:
        logger.exception("Unable to complete")
    else:
        crawler.clear(True)
        logger.info("Crawling completed")


def run():
    parser = argparse.ArgumentParser()
    parser.add_argument('spider', help='spider directory name')
    parser.add_argument('--resume', '-r', action='store_true',
                        help="resume crawl")
    parser.add_argument('-v', '--version', action='version',
                        version='%(prog)s {version}'.format(version=__version__))
    args = parser.parse_args()
    path = os.path.abspath(args.spider)
    spider_module, settings_module = load_modules(path)
    if args.resume:
        try:
            settings_module.CRAWL['RESUME'] = True
        except AttributeError:
            settings_module.CRAWL = {'RESUME': True}
    main(spider_module.Spider, settings_module)

if __name__ == "__main__":
    run()



from dragline import __version__
try:
    from cPickle import Pickler, Unpickler, HIGHEST_PROTOCOL
except:
    from pickle import Pickler, Unpickler, HIGHEST_PROTOCOL
import re
from .defaultsettings import CrawlSettings, RequestSettings
from .defaultsettings import SpiderSettings, LogSettings
from . import redisds
from gevent.coros import BoundedSemaphore
from .http import Request, RequestError
from uuid import uuid4
from datetime import datetime
from pytz import timezone
import logging
import sys
try:
    from cStringIO import StringIO
except ImportError:
    from requests.compat import StringIO


class Pickle():

    def dumps(self, obj, protocol=HIGHEST_PROTOCOL):
        file = StringIO()
        Pickler(file, protocol).dump(obj)
        return file.getvalue()

    def loads(self, str):
        file = StringIO(str)
        return Unpickler(file).load()


class Crawler:
    settings = CrawlSettings()

    def __init__(self, spider_class, settings):
        def get(value, default={}):
            try:
                return getattr(settings, value)
            except AttributeError:
                return default
        self.settings = CrawlSettings(get('CRAWL'))
        Request.settings = RequestSettings(get('REQUEST'))
        spider_settings = SpiderSettings(get('SPIDER'))
        spider = spider_class(spider_settings)
        Request.callback_object = spider
        log = LogSettings(get('LOGFORMATTERS'), get('LOGHANDLERS'),
                          get('LOGGERS'))
        if hasattr(self.settings, 'NAMESPACE'):
            logger = log.getLogger(str(self.settings.NAMESPACE))
            logger = logging.LoggerAdapter(logger, {"spider_name": spider.name})
        else:
            logger = log.getLogger(spider.name)
        spider.logger = logger
        self.logger = logger
        self.load(spider)
        Request.stats = self.stats

    def load(self, spider):
        redis_args = dict(host=self.settings.REDIS_URL,
                          port=self.settings.REDIS_PORT,
                          db=self.settings.REDIS_DB)
        if hasattr(self.settings, 'NAMESPACE'):
            redis_args['namespace'] = self.settings.NAMESPACE
        else:
            redis_args['namespace'] = spider.name
        self.url_set = redisds.Set('urlset', **redis_args)
        self.url_queue = redisds.Queue('urlqueue', serializer=Pickle(),
                                       **redis_args)
        self.runner = redisds.Lock("runner:%s" % uuid4().hex, **redis_args)
        self.runners = redisds.Dict("runner:*", **redis_args)
        self.stats = redisds.Dict("stats:*", **redis_args)
        self.lock = BoundedSemaphore(1)
        self.running_count = 0
        if not hasattr(spider, 'allowed_domains'):
            spider.allowed_domains = []
        self.allowed_urls_regex = self.get_regex(spider.allowed_domains)
        self.spider = spider
        self.start()

    def get_regex(self, domains):
        default = (r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+[A-Z]{2,6}\.?|'  # domain...
                   r'localhost|'  # localhost...
                   r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'  # ...or ip
                   r'(?::\d+)?')
        domain_regex = r'(%s)' % '|'.join(domains) if len(domains) else default
        url_regex = r'^https?://%s(?:/?|[/?]\S+)$' % domain_regex
        regex = re.compile(url_regex, re.IGNORECASE)
        return regex

    def current_time(self):
        tz = timezone(self.settings.TIME_ZONE)
        return datetime.now(tz).isoformat()

    def start(self):
        if not self.settings.RESUME and self.completed():
            self.url_queue.clear()
            self.url_set.clear()
        if self.url_queue.empty():
            self.stats.clear()
        if isinstance(self.spider.start, list):
            requests = self.spider.start
        else:
            requests = [self.spider.start]
        for request in requests:
            if isinstance(request, str):
                request = Request(request)
            if request.callback is None:
                request.callback = "parse"
            self.insert(request)
        if self.stats['status'] != "running":
            self.stats['status'] = "running"
            self.stats['start_time'] = self.current_time()
            self.logger.info("Starting spider %s", dict(self.stats))
        else:
            self.logger.info("Supporting %s", dict(self.stats))

    def clear(self, finished):
        self.runner.release()
        if self.stats['status'] != "running":
            return
        if self.completed():
            self.stats['end_time'] = self.current_time()
            if finished:
                self.stats['status'] = 'finished'
                self.url_queue.clear()
                self.url_set.clear()
            else:
                self.stats['status'] = 'stopped'
        self.logger.info("%s", dict(self.stats))

    def completed(self):
        return len(self.runners) == 0

    def inc_count(self):
        self.lock.acquire()
        if self.running_count == 0:
            self.runner.acquire()
        self.running_count += 1
        self.lock.release()

    def decr_count(self):
        self.lock.acquire()
        self.running_count -= 1
        if self.running_count == 0:
            self.runner.release()
        self.lock.release()

    def insert(self, request, check=True):
        if not isinstance(request, Request):
            return
        reqhash = request.get_unique_id(True)
        if check:
            check = not request.dontfilter
        if check:
            if not self.allowed_urls_regex.match(request.url):
                return
            elif self.settings.UNIQUE_CHECK and reqhash in self.url_set:
                return
        if self.settings.UNIQUE_CHECK and check:
            self.url_set.add(reqhash)
        self.url_queue.put(request)
        del request

    def process_url(self):
        while True:
            request = self.url_queue.get(timeout=2)
            if request:
                self.logger.debug("Processing %s", request)
                self.inc_count()
                try:
                    response = request.send()
                    try:
                        requests = request.callback(response)
                        if requests:
                            for i in requests:
                                self.insert(i)
                    except KeyboardInterrupt:
                        raise KeyboardInterrupt
                    except RequestError as e:
                        raise RequestError(e)
                    except:
                        self.logger.exception("Failed to execute callback on %s", request)
                except RequestError as e:
                    request.retry += 1
                    if request.retry >= self.settings.MAX_RETRY:
                        self.logger.warning("Rejecting %s", request, exc_info=True)
                    else:
                        self.logger.debug("Retrying %s", request, exc_info=True)
                        self.insert(request, False)
                # except Exception as e:
                # self.logger.exception('Failed to open the url %s', request)
                except KeyboardInterrupt:
                    self.insert(request, False)
                    raise KeyboardInterrupt
                else:
                    self.logger.debug("Finished processing %s", request)
                finally:
                    self.decr_count()
            else:
                if self.completed():
                    break
                self.logger.debug("No url to process, active threads: %s", self.running_count)



from dragline.http import Request
from dragline.htmlparser import HtmlParser
import argparse
from lxml.html import open_in_browser


def start_python_console(namespace=None, noipython=False, banner=''):
    """Start Python console binded to the given namespace. If IPython is
    available, an IPython console will be started instead, unless `noipython`
    is True. Also, tab completion will be used on Unix systems.
    """
    if namespace is None:
        namespace = {}

    try:
        try:  # use IPython if available
            if noipython:
                raise ImportError()
            try:
                try:
                    from IPython.terminal import embed
                except ImportError:
                    from IPython.frontend.terminal import embed
                sh = embed.InteractiveShellEmbed(banner1=banner)
            except ImportError:
                from IPython.Shell import IPShellEmbed
                sh = IPShellEmbed(banner=banner)
            sh(global_ns={}, local_ns=namespace)
        except ImportError:
            import code
            try:  # readline module is only available on unix systems
                import readline
            except ImportError:
                pass
            else:
                import rlcompleter
                readline.parse_and_bind("tab:complete")
            code.interact(banner=banner, local=namespace)
    except SystemExit:  # raised when using exit() in python code.interact
        pass


def shelp():
    repr_data = {k: repr(v) for k, v in data.iteritems()}
    intro = """\n[d] Available Dragline objects:
    [d]   parser                 %(parser)s
    [d]   request                %(request)s
    [d]   response               %(response)s
    [d] Useful shortcuts: ## Override methods in Cmd object ##
    [d]   shelp()                Shell help (print this help)
    [d]   fetch(req_or_url)      Fetch request (or URL) and update local objects
    [d]   view(response=None)    View response in a browser\n\n""" % repr_data
    print(intro)


def fetch(req_or_url):
    global data
    if isinstance(req_or_url, Request):
        data["request"] = req_or_url
    else:
        data["request"] = Request(req_or_url)
    data["response"] = data["request"].send()
    data["parser"] = HtmlParser(data["response"])
    shelp()


def view(response=None):
    if response is None:
        global data
        response = data["response"]
    open_in_browser(HtmlParser(response), 'utf-8')

data = {}
data["fetch"] = fetch
data["view"] = view
data["shelp"] = shelp
data["Request"] = Request


def execute():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        'url', action='store', default='', help='url')
    url = (parser.parse_args()).url
    fetch(url)
    start_python_console(data)


if __name__ == "__main__":
    execute()




import logging
import logging.config


class Settings:

    def __init__(self, data={}):
        self.__dict__.update(data)


class RequestSettings(Settings):
    HEADERS = {
        "user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/33.0.1750.152 Safari/537.36",
        "accept": "text/html",
        'Connection': 'close'
    }
    AUTOTHROTTLE = False
    CACHE = None
    TIMEOUT = 5
    DELAY = 0.5
    MIN_DELAY = 0.5
    MAX_DELAY = 60
    PROXIES = []

    COOKIE = True
    MAX_REDIRECTS = 1


class LogSettings:
    version = 1
    disable_existing_loggers = True
    formatters = {
        'standard': {
            'format': '%(asctime)s [%(levelname)s] %(name)s: %(message)s'
        },
    }
    handlers = {
        'default': {
            'level': 'INFO',
            'class': 'logging.StreamHandler',
            'formatter': 'standard'
        }
    }
    loggers = {
        '': {
            'handlers': ['default'],
            'level': 'INFO',
            'propagate': False
        },
        'dragline': {
            'handlers': ['default'],
            'level': 'INFO',
            'propagate': False
        }
    }

    def __init__(self, formatters={}, handlers={}, loggers={}):
        self.__update(self.formatters, formatters)
        self.__update(self.handlers, handlers)
        self.__update(self.loggers, loggers)

    def __update(self, current, new):
        for k in new:
            if k in current:
                current[k].update(new[k])
            else:
                current[k] = new[k]

    def conf(self):
        attrs = ['version', 'disable_existing_loggers', 'handlers', 'loggers',
                 'formatters']
        return {attr: getattr(self, attr) for attr in attrs}

    def getLogger(self, name=''):
        if name not in self.loggers:
            self.loggers[name] = self.loggers['']
        logging.config.dictConfig(self.conf())
        logger = logging.getLogger(name=name)
        return logger


class CrawlSettings(Settings):
    TIME_ZONE = 'UTC'
    RESUME = False
    MAX_RETRY = 3
    REDIS_URL = 'localhost'
    REDIS_PORT = 6379
    REDIS_DB = 1
    UNIQUE_CHECK = True


class SpiderSettings(Settings):
    DB = None




from glob import glob
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(name='Dragline',
      version=open('dragline/VERSION').read().strip(),
      description='Distributed Python web crawling framework',
      author='Ashwin Rajeev, Shimil Rahman',
      author_email='ashwin@inzyte.com, shimil@inzyte.com',
      url='http://www.inzyte.com',
      packages=['dragline'],
      scripts=glob("scripts/*"),
      data_files=[('dragline/templates', glob("dragline/templates/*"))],
      include_package_data=True,
      install_requires=[i.strip() for i in open('requirements.txt')],
      test_suite='tests',
      classifiers=[
          'Programming Language :: Python',
          'Programming Language :: Python :: 2.7',
          'License :: OSI Approved :: BSD License',
          'Operating System :: POSIX :: Linux',
          'Development Status :: 4 - Beta',
          'Intended Audience :: Developers',
          'Environment :: Console',
          'Topic :: Software Development :: Libraries :: Application Frameworks',
          'Topic :: Software Development :: Libraries :: Python Modules',
          'Topic :: Internet :: WWW/HTTP',
      ]
      )



from string import Template
import dragline
import ConfigParser
import os
import inspect
from httplib2 import Http
from urllib import urlencode
import base64
from runner import load_modules
import argparse
import zipfile
import pkgutil
from defaultsettings import SpiderSettings
from getpass import getpass
import stat

config_file = os.path.expanduser('~/.dragline')


def zipdir(source, destination):
    folder = os.path.abspath(source)
    with zipfile.ZipFile(destination, 'w') as zipf:
        for root, dirs, files in os.walk(folder):
            path = os.path.relpath(root, folder)
            for filename in files:
                relname = os.path.join(path, filename)
                absname = os.path.join(root, filename)
                if not filename.endswith(".pyc"):
                    zipf.write(absname, relname, zipfile.ZIP_DEFLATED)


def upload(url, username, password, foldername, spider_website=None):
    # check whether the folder is a spider
    if not "main.py" in os.listdir(foldername):
        return "Not a valid spider"

    # check if the main.py contain a spider class
    module, settings = load_modules(foldername)
    # check if main.py contain a spider class
    try:
        spider = getattr(module, "Spider")
    except Exception as e:
        return "Spider class not found"

    if not inspect.isclass(spider):
        return "Spider class not found"

    def get(value, default={}):
        try:
            return getattr(settings, value)
        except AttributeError:
            return default
    # create a spider object and check whether it contain required attributes
    settings = SpiderSettings(get('SPIDER'))
    spider_object = spider(settings)

    try:
        if spider_object.name and spider_object.start:
            spider_name = spider_object.name
        else:
            return "required attributes not found in spider"

    except Exception as e:
        print e
        return "Spider deploying failed"

    # zip the folder
    zipdir(foldername, "/tmp/%s.zip" % spider_name)
    zipf = base64.encodestring(open("/tmp/%s.zip" % spider_name, "rb").read())
    post_data = {'username': username, 'password': password, 'name':
                 spider_name, 'zipfile': zipf}
    if spider_website:
        post_data['website'] = spider_website
    headers = {'content-type': 'application/x-www-form-urlencoded'}
    h = Http()
    resp, content = h.request(
        url, "POST", body=urlencode(post_data), headers=headers)
    # read zip file
    return content


def deploy(serv_name, spider_dir):
    parser = ConfigParser.SafeConfigParser()
    parser.read(config_file)
    try:
        args = dict(parser.items(serv_name))
    except ConfigParser.NoSectionError:
        add_server(serv_name)
        deploy(serv_name, spider_dir)
        return
    args['foldername'] = spider_dir
    print upload(**args)


def generate_file(name, data, destination, executable=False):
    tem = Template(pkgutil.get_data(__package__, "templates/%s.tem" % name))
    content = tem.substitute(**data)
    filename = os.path.join(destination, "%s.py" % name)
    with open(filename, "w") as outfile:
        outfile.write(content)
    if executable:
        mode = os.stat(filename).st_mode
        os.chmod(filename, mode | stat.S_IXUSR)


def generate(spider_dir):
    os.makedirs(spider_dir)
    spider_name = os.path.basename(os.path.normpath('dragline/templates/'))
    generate_file("main", {'spider_name': spider_name}, spider_dir, True)
    generate_file("settings", {}, spider_dir)
    generate_file("db", {}, spider_dir)


def add_server(serv_name):
    url = raw_input("Enter URL:")
    usr_name = raw_input("Enter Username:")
    pwd = getpass("Enter the password:")
    parser = ConfigParser.SafeConfigParser()
    parser.read(config_file)
    if not parser.has_section(serv_name):
        parser.add_section(serv_name)
    parser.set(serv_name, 'url', url)
    parser.set(serv_name, 'username', usr_name)
    parser.set(serv_name, 'password', pwd)
    parser.write(open(config_file, 'w'))


def execute():
    parser = argparse.ArgumentParser(description='Dragline commandparse')
    subparsers = parser.add_subparsers(
        title='subcommands', description='valid subcommands', help='additional help')

    parser_first = subparsers.add_parser('init')
    parser_first.set_defaults(which='init')
    parser_first.add_argument('spider', help='spider name')

    parser_second = subparsers.add_parser('deploy')
    parser_second.set_defaults(which='deploy')
    parser_second.add_argument('server', help='server name')
    parser_second.add_argument('spider', help='spider name')

    parser_third = subparsers.add_parser('addserver')
    parser_third.set_defaults(which='addserver')
    parser_third.add_argument('server', help='assign work for a server')

    args = vars(parser.parse_args())

    if args['which'] == 'init':
        generate(args['spider'])
    elif args['which'] == 'deploy':
        deploy(args['server'], args['spider'])
    elif args['which'] == 'addserver':
        add_server(args['server'])


if __name__ == "__main__":
    execute()
